@@ -410,6 +410,17 @@ pred_delete (const char *pathname, struct stat *stat_buf, struct predicate *pred
 	}
       error (0, errno, _("cannot delete %s"),
 	     safely_quote_err_filename(0, pathname));
+      /* Previously I had believed that having the -delete action
+       * return false provided the user with control over whether an
+       * error message is issued.  While this is true, the policy of
+       * not affecting the exit status is contrary to the POSIX
+       * requirement that diagnostic messages are accompanied by a
+       * nonzero exit status.  While -delete is not a POSIX option and
+       * we can therefore opt not to follow POSIX in this case, that
+       * seems somewhat arbitrary and confusing.  So, as of
+       * findutils-4.3.11, we also set the exit status in this case.
+       */
+      state.exit_status = 1;
       return false;
     }
   else
@@ -1156,29 +1167,28 @@ pred_ilname (const char *pathname, struct stat *stat_buf, struct predicate *pred
 
 /* Common code between -name, -iname.  PATHNAME is being visited, STR
    is name to compare basename against, and FLAGS are passed to
-   fnmatch.  */
+   fnmatch.  Recall that 'find / -name /' is one of the few times where a '/' 
+   in the -name must actually find something. */ 
 static boolean
 pred_name_common (const char *pathname, const char *str, int flags)
 {
-  /* Prefer last_component over base_name, to avoid malloc when
-     possible.  */
-  char *base = last_component (pathname);
+  char *p;
+  boolean b;
+  /* We used to use last_component() here, but that would not allow us to modify the 
+   * input string, which is const.   We could optimise by duplicating the string only
+   * if we need to modify it, and I'll do that if there is a measurable 
+   * performance difference on a machine built after 1990...
+   */
+  char *base = base_name (pathname);
+  /* remove trailing slashes, but leave  "/" or "//foo" unchanged. */
+  strip_trailing_slashes(base);
 
-  /* base is empty only if pathname is a file system root.  But recall
-     that 'find / -name /' is one of the few times where a '/' in the
-     -name must actually find something.  */
-  if (!*base)
-    {
-      boolean b;
-      base = base_name (pathname);
-      b = fnmatch (str, base, flags) == 0;
-      free (base);
-      return b;
-    }
   /* FNM_PERIOD is not used here because POSIX requires that it not be.
    * See http://standards.ieee.org/reading/ieee/interp/1003-2-92_int/pasc-1003.2-126.html
    */
-  return fnmatch (str, base, flags) == 0;
+  b = fnmatch (str, base, flags) == 0;
+  free (base);
+  return b;
 }
 
 boolean
@@ -1582,12 +1592,16 @@ pred_prune (const char *pathname, struct stat *stat_buf, struct predicate *pred_
   (void) pathname;
   (void) pred_ptr;
 
-  if (options.do_dir_first == true &&
+  if (options.do_dir_first == true && /* no effect with -depth */
       stat_buf != NULL &&
       S_ISDIR(stat_buf->st_mode))
     state.stop_at_current_level = true;
 
-  return (options.do_dir_first); /* This is what SunOS find seems to do. */
+  /* findutils used to return options.do_dir_first here, so that -prune
+   * returns true only if -depth is not in effect.   But POSIX requires 
+   * that -prune always evaluate as true.
+   */
+  return true;
 }
 
 boolean
