@@ -2,20 +2,18 @@
    Copyright (C) 1994, 1996, 1998, 1999, 2000, 2003,
                  2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Usage: locate [options] pattern...
@@ -30,8 +28,8 @@
 
    0-28		likeliest differential counts + offset (14) to make nonnegative
    30		escape code for out-of-range count to follow in next halfword
-   128-255 	bigram codes (the 128 most common, as determined by `updatedb')
-   32-127  	single character (printable) ASCII remainder
+   128-255      bigram codes (the 128 most common, as determined by `updatedb')
+   32-127       single character (printable) ASCII remainder
 
    Earlier versions of GNU locate used to use a novel two-tiered
    string search technique, which was described in Usenix ;login:, Vol
@@ -81,10 +79,7 @@
 /* We need <unistd.h> for isatty(). */
 #include <unistd.h>
 
-#if HAVE_FCNTL_H
-/* We use fcntl() */
 #include <fcntl.h>
-#endif
 
 #define NDEBUG
 #include <assert.h>
@@ -123,7 +118,6 @@
 #endif
 
 #include "locatedb.h"
-#include <getline.h>
 #include "xalloc.h"
 #include "error.h"
 #include "human.h"
@@ -135,7 +129,7 @@
 #include "quotearg.h"
 #include "printquoted.h"
 #include "regextype.h"
-#include "gnulib-version.h"
+#include "findutils-version.h"
 
 /* Note that this evaluates Ch many times.  */
 #ifdef _LIBC
@@ -200,7 +194,7 @@ set_max_db_age(const char *s)
   if (0 == *s)
     {
       error(1, 0,
-	    _("The argument argument for option --max-database-age must not be empty"));
+	    _("The argument for option --max-database-age must not be empty"));
     }
   
   
@@ -342,7 +336,7 @@ struct process_data
   int len;
   char *original_filename;	/* The current input database entry. */
   size_t pathsize;		/* Amount allocated for it.  */
-  char *munged_filename;	/* path or base_name(path) */
+  char *munged_filename;	/* path or basename(path) */
   FILE *fp;			/* The pathname database.  */
   const char *dbfile;		/* Its name, or "<stdin>" */
   int  slocatedb_format;	/* Allows us to cope with slocate's format variant */
@@ -646,7 +640,7 @@ static int
 visit_basename(struct process_data *procdata, void *context)
 {
   (void) context;
-  procdata->munged_filename = base_name(procdata->original_filename);
+  procdata->munged_filename = last_component (procdata->original_filename);
 
   return VISIT_CONTINUE;
 }
@@ -1062,7 +1056,7 @@ search_one_database (int argc,
 		     int regex,
 		     int regex_options)
 {
-  char *pathpart; 		/* A pattern to consider. */
+  char *pathpart;		/* A pattern to consider. */
   int argn;			/* Index to current pattern in argv. */
   int nread;		     /* number of bytes read from an entry. */
   struct process_data procdata;	/* Storage for data shared with visitors. */
@@ -1187,10 +1181,10 @@ search_one_database (int argc,
 	  int i;
 
 	  nread += nread2;
+	  extend (&procdata, 256u, 0u);
 	  /* Read the list of the most common bigrams in the database.  */
 	  if (nread < 256)
 	    {
-	      extend (&procdata, 256u, 0u);
 	      int more_read = fread (procdata.original_filename + nread, 1,
 				     256 - nread, procdata.fp);
 	      if ( (more_read + nread) != 256 )
@@ -1231,7 +1225,7 @@ search_one_database (int argc,
 	  
 	  re_set_syntax(regex_options);
 	  p->regex.allocated = 100;
-	  p->regex.buffer = (unsigned char *) xmalloc (p->regex.allocated);
+	  p->regex.buffer = xmalloc (p->regex.allocated);
 	  p->regex.fastmap = NULL;
 	  p->regex.syntax = regex_options;
 	  p->regex.translate = NULL;
@@ -1550,7 +1544,7 @@ static int
 opendb(const char *name)
 {
   int fd = open(name, O_RDONLY
-#if defined(O_LARGEFILE)
+#if defined O_LARGEFILE
 		|O_LARGEFILE
 #endif
 		);
@@ -1571,7 +1565,6 @@ dolocate (int argc, char **argv, int secure_db_fd)
 {
   char *dbpath;
   unsigned long int found = 0uL;
-  int optc;
   int ignore_case = 0;
   int print = 0;
   int just_count = 0;
@@ -1613,114 +1606,120 @@ dolocate (int argc, char **argv, int secure_db_fd)
 
   check_existence = ACCEPT_EITHER;
 
-  while ((optc = getopt_long (argc, argv, "Abcd:eEil:prsm0SwHPL", longopts, (int *) 0)) != -1)
-    switch (optc)
-      {
-      case '0':
-	separator = 0;
-	print_quoted_filename = false; /* print filename 'raw'. */
+  for (;;)
+    {
+      int opti = -1;
+      int optc = getopt_long (argc, argv, "Abcd:eEil:prsm0SwHPL", longopts,
+			      &opti);
+      if (optc == -1)
 	break;
 
-      case 'A':
-	op_and = 1;
-	break;
-
-      case 'b':
-	basename_only = 1;
-	break;
-
-      case 'c':
-	just_count = 1;
-	break;
-
-      case 'd':
-	dbpath = optarg;
-	they_chose_db = 1;
-	break;
-
-      case 'e':
-	check_existence = ACCEPT_EXISTING;
-	break;
-
-      case 'E':
-	check_existence = ACCEPT_NON_EXISTING;
-	break;
-
-      case 'i':
-	ignore_case = 1;
-	break;
-	
-      case 'h':
-	usage (stdout);
-	return 0;
-
-      case MAX_DB_AGE:
-	/* XXX: nothing in the test suite for this option. */
-	set_max_db_age(optarg);
-	break;
-
-      case 'p':
-	print = 1;
-	break;
-
-      case 'v':
-	printf (_("GNU locate version %s\n"), version_string);
-	printf (_("Built using GNU gnulib version %s\n"), gnulib_version);
-	return 0;
-
-      case 'w':
-	basename_only = 0;
-	break;
-
-      case 'r':
-	regex = 1;
-	break;
-
-      case REGEXTYPE_OPTION:
-	regex_options = get_regex_type(optarg);
-	break;
-	
-      case 'S':
-	stats = 1;
-	break;
-
-      case 'L':
-	follow_symlinks = 1;
-	break;
-
-	/* In find, -P and -H differ in the way they handle paths
-	 * given on the command line.  This is not relevant for
-	 * locate, but the -H option is supported because it is
-	 * probably more intuitive to do so.
-	 */
-      case 'P':
-      case 'H':
-	follow_symlinks = 0;
-	break;
-
-      case 'l':
+      switch (optc)
 	{
-	  char *end = optarg;
-	  strtol_error err = xstrtoumax(optarg, &end, 10, &limits.limit, NULL);
-	  if (LONGINT_OK != err)
-	    {
-	      STRTOL_FATAL_ERROR(optarg, _("argument to --limit"), err);
-	    }
-	  use_limit = 1;
+	case '0':
+	  separator = 0;
+	  print_quoted_filename = false; /* print filename 'raw'. */
+	  break;
+
+	case 'A':
+	  op_and = 1;
+	  break;
+
+	case 'b':
+	  basename_only = 1;
+	  break;
+
+	case 'c':
+	  just_count = 1;
+	  break;
+
+	case 'd':
+	  dbpath = optarg;
+	  they_chose_db = 1;
+	  break;
+
+	case 'e':
+	  check_existence = ACCEPT_EXISTING;
+	  break;
+
+	case 'E':
+	  check_existence = ACCEPT_NON_EXISTING;
+	  break;
+
+	case 'i':
+	  ignore_case = 1;
+	  break;
+
+	case 'h':
+	  usage (stdout);
+	  return 0;
+
+	case MAX_DB_AGE:
+	  /* XXX: nothing in the test suite for this option. */
+	  set_max_db_age (optarg);
+	  break;
+
+	case 'p':
+	  print = 1;
+	  break;
+
+	case 'v':
+	  display_findutils_version ("locate");
+	  return 0;
+
+	case 'w':
+	  basename_only = 0;
+	  break;
+
+	case 'r':
+	  regex = 1;
+	  break;
+
+	case REGEXTYPE_OPTION:
+	  regex_options = get_regex_type (optarg);
+	  break;
+
+	case 'S':
+	  stats = 1;
+	  break;
+
+	case 'L':
+	  follow_symlinks = 1;
+	  break;
+
+	  /* In find, -P and -H differ in the way they handle paths
+	   * given on the command line.  This is not relevant for
+	   * locate, but the -H option is supported because it is
+	   * probably more intuitive to do so.
+	   */
+	case 'P':
+	case 'H':
+	  follow_symlinks = 0;
+	  break;
+
+	case 'l':
+	  {
+	    char *end = optarg;
+	    strtol_error err = xstrtoumax (optarg, &end, 10, &limits.limit,
+					   NULL);
+	    if (LONGINT_OK != err)
+	      xstrtol_fatal (err, opti, optc, longopts, optarg);
+	    use_limit = 1;
+	  }
+	  break;
+
+	case 's':			/* use stdio */
+	case 'm':			/* use mmap  */
+	  /* These options are implemented simply for
+	   * compatibility with FreeBSD
+	   */
+	  break;
+
+	default:
+	  usage (stderr);
+	  return 1;
 	}
-	break;
-
-      case 's':			/* use stdio */
-      case 'm':			/* use mmap  */
-	/* These options are implemented simply for
-	 * compatibility with FreeBSD
-	 */ 
-	break;
-
-      default:
-	usage (stderr);
-	return 1;
-      }
+    }
 
 
   /* If the user gave the -d option or set LOCATE_PATH,
@@ -1741,7 +1740,7 @@ dolocate (int argc, char **argv, int secure_db_fd)
   if (stats)
     {
       if (optind == argc)
- 	  use_limit = 0;
+	use_limit = 0;
     }
   else
     {
@@ -1849,7 +1848,7 @@ dolocate (int argc, char **argv, int secure_db_fd)
 	       * database is.  But since the message is just advisory,
 	       * we continue anyway.
 	       */
-	      error (0, errno, "time system call");
+	      error (0, errno, _("time system call failed"));
 	    }
 	  else
 	    {
@@ -1935,4 +1934,3 @@ main (int argc, char **argv)
   
   return dolocate(argc, argv, dbfd);
 }
-
